import { ChangeDetectorRef, Component, EventEmitter, Input, OnInit, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { EmpGroupModel, MyEmpGroupModel } from 'src/app/shared/model/emp-group.model';
import { EmpGroupService } from 'src/app/shared/services/emp-group.service';
import { FileService } from 'src/app/shared/services/file.service';
export interface DataModel {
  groupId: string
  tdesc: string
  edesc: string
  companyId: string
  checked: boolean
}

@Component({
  selector: 'app-employee-group-unit',
  templateUrl: './employee-group-unit.component.html',
  styleUrls: ['./employee-group-unit.component.scss']
})
export class EmployeeGroupUnit implements OnInit {
  currentPage = 1
  selectedItems: string[] = [];
  modalStatus: 'add' | 'edit' | 'delete' | 'deleteGroup' = 'add'
  page = Array.from({ length: 1 }, (_, i) => i + 1);
  dataLoading = false
  dataSelect: DataModel = { groupId: "", tdesc: "", edesc: "", companyId: "", checked: false }
  emp_groupList: { check: boolean; data: DataModel & { checked?: boolean } }[] = []
  emp_group: EmpGroupModel = new MyEmpGroupModel({})
  search = ""
  selectedFile: File | null = null;
  selectedFileName: string = 'กรุณาเลือกไฟล์';

  numDataListChecked = 0
  isDataListChecked = false
  isDataListCheckedAll = false
  constructor(private empGroupService: EmpGroupService,
    private toastr: ToastrService,
    private cdr: ChangeDetectorRef,
    private fileService: FileService
  ) { }

  ngOnInit(): void {
    this.getEmpGroupList()
  }

  onFileSelected(event: any) {
    this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
    this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
  }

  uploadFile() {
    if (!this.selectedFile) {
      alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
      return
    }
    const formData = new FormData();
    formData.append('file', this.selectedFile);
    this.dataLoading = true
    this.fileService.upload(formData, 'mgroup').subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getEmpGroupList()
        } else {
          this.showAlert(response.message, 'error')
          this.dataLoading = false
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
      }
    })
  }

  downloadFile() {
    const fileName = 'IMPORT_MGROUP.xlsx'
    this.fileService.download(fileName).subscribe({
      next: response => {
        const url = window.URL.createObjectURL(response);
        const a = document.createElement("a");
        a.href = url;
        a.download = fileName;
        document.body.appendChild(a);
        a.click();
        document.body.removeChild(a);
        window.URL.revokeObjectURL(url);
      }, error: error => {
        this.showAlert(error.message, 'error')
      }
    })

  }

  getEmpGroupList() {
    this.dataLoading = true
    this.empGroupService.getList().subscribe({
      next: response => {
        this.emp_groupList = response.map(x => ({ check: false, data: { groupId: x.groupId, tdesc: x.tdesc, edesc: x.edesc, companyId: x.companyId, checked: false } }))
        this.dataLoading = false
        this.searchChange()
        this.isDataListCheckedAll = false
        this.dataListCheckAll()
        this.cdr.detectChanges()
      }, error: error => {
        this.dataLoading = false
        console.error('Error fetching employee types:', error);
        this.cdr.detectChanges()
      }
    })
  }
  searchChange() {
    this.currentPage = 1
    this.page = Array.from({ length: Math.ceil(this.emp_groupListFilter().length / 10) }, (_, i) => i + 1);
    this.dataListCheck()
  }
  emp_groupListFilter() {
    return this.emp_groupList.filter(x => {
      const data = x.data
      const match = data.groupId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase()) || data.edesc.toLowerCase().includes(this.search.toLowerCase());
      return match;
    });
  }
  setData(data?: DataModel) {
    this.dataSelect = JSON.parse(JSON.stringify(data || { groupId: "", tdesc: "", edesc: "", companyId: "" }));
  }

  addEmp_group() {
    const body = new MyEmpGroupModel({ groupId: this.dataSelect.groupId, tdesc: this.dataSelect.tdesc, edesc: this.dataSelect.edesc, companyId: this.dataSelect.companyId })
    this.dataLoading = true
    this.empGroupService.post(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success')
          this.getEmpGroupList()
        } else {
          this.showAlert(response.message, 'error')
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      }, error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    })
  }
  deleteEmp_group() {
    let body: EmpGroupModel | EmpGroupModel[] = [];
    if (this.dataSelect.groupId) {
      body = new MyEmpGroupModel({ groupId: this.dataSelect.groupId, tdesc: this.dataSelect.tdesc, edesc: this.dataSelect.edesc, companyId: this.dataSelect.companyId });
    } else {
      body = this.emp_groupList.filter(x => x.check).map(x => new MyEmpGroupModel({ groupId: x.data.groupId, tdesc: x.data.tdesc, edesc: x.data.edesc, companyId: x.data.companyId }));
    }
    this.dataLoading = true
    this.empGroupService.delete(body).subscribe({
      next: response => {
        if (response.success) {
          this.showAlert(response.message, 'success');
          this.getEmpGroupList();
        } else {
          this.showAlert(response.message, 'error');
          this.dataLoading = false
          this.cdr.detectChanges()
        }
      },
      error: error => {
        this.showAlert(error.message, 'error')
        this.dataLoading = false
        this.cdr.detectChanges()
      }
    });
  }

  dataListCheckAll() {
    const selectAll = this.isDataListCheckedAll;
    this.emp_groupList.filter(x => {
      const data = x.data
      const match = data.groupId.toLowerCase().includes(this.search.toLowerCase()) || data.tdesc.toLowerCase().includes(this.search.toLowerCase()) || data.edesc.toLowerCase().includes(this.search.toLowerCase());
      return match;
    }).forEach(x => x.check = selectAll);
    this.dataListCheck();
  }

  dataListCheck() {
    const dataCheck = this.emp_groupListFilter();
    this.isDataListCheckedAll = dataCheck.length ? dataCheck.every(x => x.check) : false;
    this.numDataListChecked = this.emp_groupList.filter(x => x.check).length;
    this.isDataListChecked = Boolean(this.numDataListChecked)
  }

  showAlert(text: string, type: 'success' | 'error') {
    this.toastr[type](text, 'แจ้งเตือน', {
      timeOut: 3000,
      positionClass: 'toast-top-right',
    })
  }

  clearEmp_group(modalStatus: string) {
    if (modalStatus == 'add') {
      this.dataSelect.groupId = ''
      this.dataSelect.tdesc = ''
      this.dataSelect.edesc = ''
    } else if (modalStatus == 'edit') {
      this.dataSelect.tdesc = ''
      this.dataSelect.edesc = ''
    }
  }
}
