import { ChangeDetectorRef, Component, EventEmitter, Input, Output } from '@angular/core';
import { ToastrService } from 'ngx-toastr';
import { JobCodeModel, MyJobCodeModel } from 'src/app/shared/model/job-code.model';
import { JobModel, MyJobModel } from 'src/app/shared/model/job.model';
import { FileService } from 'src/app/shared/services/file.service';
import { JobCodeService } from 'src/app/shared/services/่job-code.service';


@Component({
  selector: 'app-sub-job-competency',
  templateUrl: './sub-job-competency.component.html',
  styleUrls: ['./sub-job-competency.component.scss']
})
export class SubJobCompetencyComponent {
    currentPage = 1
    page = Array.from({ length: 1 }, (_, i) => i + 1);
    search = ""
    jobCodeList:JobCodeModel[]=[]
    loading = false

    selectedFile: File | null = null;
    selectedFileName: string = 'กรุณาเลือกไฟล์';
    constructor(
      private toastr: ToastrService,
      private fileService: FileService,
      private jobcodeService: JobCodeService,
       private cdr: ChangeDetectorRef,
    ) { }
    ngOnInit(): void {
      this.getListJob();
    }
    getListJob(){
      this.loading = false
      this.jobcodeService.getList().subscribe({
        next: response => {
          this.jobCodeList = response.map((x: any) => new MyJobCodeModel(x))
          console.log("🚀 ~ SubJobCompetencyComponent ~ this.jobcodeService.getList ~  this.jobCodeList:",  this.jobCodeList)
          this.loading = false
          this.searchChange()
          this.cdr.detectChanges()
        }, error: error => {
          this.loading = false
          this.cdr.detectChanges()
        }
      })
    }

    searchChange() {
      this.currentPage = 1
      this.page = Array.from({ length: Math.ceil(this.jobcodeFilter().length / 10) }, (_, i) => i + 1);
    }

    jobcodeFilter() {
      return this.jobCodeList.filter(x =>
        x.tdesc.toLowerCase().includes(this.search.toLowerCase()) ||
        x.edesc.toLowerCase().includes(this.search.toLowerCase()) ||
        x.jobcodeId.toLowerCase().includes(this.search.toLowerCase())
      )
    }
    onFileSelected(event: any) {
      this.selectedFile = event.target.files.length > 0 ? event.target.files[0] : null;
      this.selectedFileName = this.selectedFile?.name || "กรุณาเลือกไฟล์"
    }
  
    uploadFile() {
      if (!this.selectedFile) {
        alert('กรุณาเลือกไฟล์ก่อนอัปโหลด')
        return
      }
      const formData = new FormData();
      formData.append('file', this.selectedFile);
      // this.bu3ListLoading = true
      this.fileService.upload(formData, 'MJOBCODE_COMPETENCY').subscribe({
        next: response => {
          if (response.success) {
            this.showAlert(response.message, 'success')
            // this.getBu3List()
          } else {
            this.showAlert(response.message, 'error')
            // this.bu3ListLoading = false
          }
        }, error: error => {
          this.showAlert(error.message, 'error')
          // this.bu3ListLoading = false
        }
      })
    }


    downloadFile() {
      const fileName = 'IMPORT_MJOBCODEV2.xlsx'
      this.fileService.download(fileName).subscribe({
        next: response => {
          const url = window.URL.createObjectURL(response);
          const a = document.createElement("a");
          a.href = url;
          a.download = fileName;
          document.body.appendChild(a);
          a.click();
          document.body.removeChild(a);
          window.URL.revokeObjectURL(url);
        }, error: error => {
          this.showAlert(error.message, 'error')
        }
      })
  
    }

    showAlert(text: string, type: 'success' | 'error') {
      this.toastr[type](text, 'แจ้งเตือน', {
        timeOut: 3000,
        positionClass: 'toast-top-right',
      })
    }

   
    showSuccessAdd() {
      this.toastr.success('บันทึกข้อมูลสำเร็จ', 'เเจ้งเตือน', {
        timeOut: 3000,
        positionClass: 'toast-top-right',
      });
    }
  
    showSuccessEdit() {
      this.toastr.success('เเก้ไขข้อมูลสำเร็จ', 'เเจ้งเตือน', {
        timeOut: 3000,
        positionClass: 'toast-top-right',
      });
    }
  
    showSuccessDelete() {
      this.toastr.success('ลบข้อมูลสำเร็จ', 'เเจ้งเตือน', {
        timeOut: 3000,
        positionClass: 'toast-top-right',
      });
    }
   
  
  }
  
